#!/usr/bin/env ruby
require 'generator.rb'
require 'enumerator.rb'
require 'iterator.rb'

$iter_classes = [Iterator, Enumerator, Generator]

class Times # record CPU & elaps times
  attr_reader :elaps, :ucpu, :scpu
  def initialize(elaps = nil, ucpu = nil, scpu = nil)
    elaps ? @elaps = elaps : @elaps = Time.now
    t = Process.times
    ucpu ? @ucpu = ucpu : @ucpu = t.utime
    scpu ? @scpu = scpu : @scpu = t.stime
  end
  def -(t)
    self.class.new(@elaps - t.elaps, @ucpu - t.ucpu, @scpu - t.scpu)
  end
  def to_s
    "#{@elaps} #{@ucpu} #{@scpu}"
  end
  def cpu
    @ucpu + @scpu
  end
end

class Result
  def initialize(num = nil, times = nil)
    @num = num; @times = times
  end
  def to_s
    "#{@num} #{@times}"
  end
end

class Bench

  def initialize(make, start, num = nil, step = nil)
    @make = make
    @start = start; @num = (num || 10); @step = (step || start) 
  end

  def run
    size = @start
    results_h = {}
    $iter_classes.each { |k| results_h[k] = [] }
    @num.times do
      enum = @make.call(size)
      @kind = enum.class
      $iter_classes.each do |iter_class|
        result = run_iter(iter_class, enum)
        results_h[iter_class].push(result)
      end
      size += @step
    end
    results_h.each do |iter_class, results|
      fn = "results/#{@kind}-#{iter_class}-#{@start}-#{@num}-#{@step}"
      File.open(fn, 'wb') do |f|
        results.each { |r|; f.puts(r) }
      end
    end
  end

private
  def run_iter(iter_class, enum)
    iterate(iter_class, enum) # just for fun
    start = Times.new
    GC.start
    #1.times { iterate(iter_class, enum) }
    puts "#{iter_class} #{enum.size} running"
    10.times { iterate(iter_class, enum) }
    times = Times.new - start
    ret = Result.new(enum.size, times)
    ret
  end

  def iterate(iter_class, enum)
    iter = iter_class.new(enum)
    v = nil
    while iter.next?
      v = iter.next
    #p v
    end
  end

end

# main object
require 'plot.rb'
include Plot

  def make_hash(size)
    hsh = {}
    v = 'aaaa'
    size.times do |i|
      hsh[v.to_sym] = v.to_sym
      v = v.succ
    end
    hsh
  end

  def make_array(size)
    ary = []
    v = 'aaaa'
    size.times do |i|
      ary.push(v.to_sym)
      v = v.succ
    end
    ary
  end

def bench_one(make)
  [1, 10, 100, 1000, 10000].each do |num|
  #[1, 10, 100].each do |num|
    Bench.new(make, num).run
  end
end

def bench_all
  bench_one(self.method(:make_array))
  bench_one(self.method(:make_hash))
end

if ARGV.empty?
  bench_all
  plot($iter_classes)
else
  make = nil
  if ARGV[0] == 'h'
    make = method(:make_hash)
  elsif ARGV[0] == 'a'
    make = method(:make_array)
  else
    raise ArgumentError
  end
  start = ARGV[1].to_i
  num = ARGV[2] ? ARGV[2].to_i : nil
  step = ARGV[3] ? ARGV[3].to_i : nil
  Bench.new(make, start, num, step).run
end
  
